export interface GameOptions {
	mode: "easy" | "hard" | "extreme demon ultra miguel";
	metro: boolean;
	tram: boolean;
}

export interface GameData {
	lines: [GeoJSON.Feature, string][];
	stops: GeoJSON.Feature[];
	stopName: string;
	gameId: string;
}

export interface CheckData {
	gameId: string;
	latlng: [number, number];
}

export interface CheckResponse {
	solution: [number, number];
	distance: number;
	score: number;
}

type FetchType = typeof fetch;

const metroUrl =
	"https://data.grandlyon.com/geoserver/sytral/ows?SERVICE=WFS&VERSION=2.0.0&request=GetFeature&typename=sytral:tcl_sytral.tcllignemf_2_0_0&outputFormat=application/json&SRSNAME=EPSG:4171&sortBy=gid";
const tramUrl =
	"https://data.grandlyon.com/geoserver/sytral/ows?SERVICE=WFS&VERSION=2.0.0&request=GetFeature&typename=sytral:tcl_sytral.tcllignetram_2_0_0&outputFormat=application/json&SRSNAME=EPSG:4171&sortBy=gid";
const stopsUrl =
	"https://data.grandlyon.com/geoserver/sytral/ows?SERVICE=WFS&VERSION=2.0.0&request=GetFeature&typename=sytral:tcl_sytral.tclarret&outputFormat=application/json&SRSNAME=EPSG:4171&sortBy=gid";

let lazyMetro: [GeoJSON.Feature, string][] | null = null;
let lazyTram: [GeoJSON.Feature, string][] | null = null;
let lazyStops: GeoJSON.Feature[] | null = null;

export async function createGame(stop: GeoJSON.Feature, kv: KVNamespace<string>): Promise<string> {
	const uuid = crypto.randomUUID();

	await kv.put(`game:${uuid}`, JSON.stringify(stop), { expirationTtl: 600 });

	return uuid;
}

export async function stopGame(
	uuid: string,
	kv: KVNamespace<string>,
): Promise<GeoJSON.Feature | null> {
	const stop: GeoJSON.Feature | null = await kv.get(`game:${uuid}`, "json");

	if (stop) await kv.delete(`game:${uuid}`);

	return stop;
}

export async function getMetro(fetch: FetchType): Promise<[GeoJSON.Feature, string][]> {
	if (!lazyMetro) {
		const metro: GeoJSON.FeatureCollection = await fetch(metroUrl).then((r) => r.json());

		lazyMetro = metro.features.map((f) => {
			const components = f.properties!.couleur!.split(" ");
			return [f, `rgb(${components.join(",")})`];
		});
	}

	return lazyMetro;
}

export async function getTram(fetch: FetchType): Promise<[GeoJSON.Feature, string][]> {
	if (!lazyTram) {
		const tram: GeoJSON.FeatureCollection = await fetch(tramUrl).then((r) => r.json());

		lazyTram = tram.features.map((f) => {
			const components = f.properties!.couleur!.split(" ");
			return [f, `rgb(${components.join(",")})`];
		});
	}

	return lazyTram;
}

export async function getStops(fetch: FetchType): Promise<GeoJSON.Feature[]> {
	if (!lazyStops) {
		const stops: GeoJSON.FeatureCollection = await fetch(stopsUrl).then((r) => r.json());

		lazyStops = Array.from(new Set(stops.features.map((f) => f.properties?.nom))).map((nom) => {
			const matching = stops.features.filter((f) => f.properties?.nom === nom);

			const longs = matching.map((f) => (f.geometry as GeoJSON.Point).coordinates[0]);
			const lonAvg = longs.reduce((a, b) => a + b) / longs.length;

			const lats = matching.map((f) => (f.geometry as GeoJSON.Point).coordinates[1]);
			const latAvg = lats.reduce((a, b) => a + b) / lats.length;

			const desserte = matching.map((f) => f.properties?.desserte).join(",");

			const feature = matching[0];
			feature.geometry = { type: "Point", coordinates: [lonAvg, latAvg] };
			if (feature.properties) feature.properties.desserte = desserte;

			return feature;
		});
	}

	return lazyStops;
}
